"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const key_1 = require("./key");
const update_expression_1 = require("./update-expression");
const condition_expression_1 = require("./condition-expression");
const expression_1 = require("./expression");
function getItemInput(params) {
    const Key = params.key;
    const TableName = params.tableName;
    const input = {
        Key,
        TableName,
    };
    if (!params.attributes || params.attributes.length === 0) {
        return input;
    }
    const projection = expression_1.buildProjectionExpression(params.attributes);
    input.ProjectionExpression = projection.expression;
    input.ExpressionAttributeNames = projection.names;
    return input;
}
exports.getItemInput = getItemInput;
function batchGetItemInput(params) {
    const KeysAndAttributes = {
        Keys: params.keys,
    };
    const RequestItems = {};
    RequestItems[params.tableName] = KeysAndAttributes;
    const input = {
        RequestItems,
    };
    if (params.attributes && params.attributes.length) {
        const projection = expression_1.buildProjectionExpression(params.attributes);
        KeysAndAttributes.ProjectionExpression = projection.expression;
        KeysAndAttributes.ExpressionAttributeNames = projection.names;
    }
    return input;
}
exports.batchGetItemInput = batchGetItemInput;
function deleteItemInput(params) {
    const Key = params.key;
    const TableName = params.tableName;
    const input = {
        Key,
        TableName,
    };
    return input;
}
exports.deleteItemInput = deleteItemInput;
function putItemInput(params) {
    const TableName = params.tableName;
    const Item = params.item;
    const input = {
        Item,
        TableName,
    };
    return input;
}
exports.putItemInput = putItemInput;
function createItemInput(params) {
    const TableName = params.tableName;
    const key = key_1.getKeyFromItem(params.item, params.hashKeyName, params.rangeKeyName);
    const Item = params.item;
    const input = {
        Item,
        TableName,
    };
    const { expression, names, values } = condition_expression_1.buildConditionExpression({
        operation: '<>',
        hashKey: {
            name: params.hashKeyName,
            value: key[params.hashKeyName],
        },
        rangeKey: params.rangeKeyName && {
            name: params.rangeKeyName,
            value: key[params.rangeKeyName],
        } || undefined,
    });
    if (expression) {
        input.ConditionExpression = expression;
    }
    if (Object.keys(names).length) {
        input.ExpressionAttributeNames = names;
    }
    if (Object.keys(values).length) {
        input.ExpressionAttributeValues = values;
    }
    return input;
}
exports.createItemInput = createItemInput;
function updateItemInput(params) {
    const TableName = params.tableName;
    const key = params.key;
    const input = {
        Key: key,
        TableName,
    };
    const keyProps = Object.keys(key);
    if (keyProps.length > 2) {
        throw new TypeError(`key must contain maximum 2 props: hash & range`);
    }
    const rangeKeyName = keyProps.length === 2 ? keyProps.find(name => name !== params.hashKeyName) : undefined;
    let { expression, names, values } = condition_expression_1.buildConditionExpression({
        operation: '=',
        hashKey: {
            name: params.hashKeyName,
            value: key[params.hashKeyName],
        },
        rangeKey: rangeKeyName && {
            name: rangeKeyName,
            value: key[rangeKeyName],
        } || undefined,
    });
    if (expression) {
        input.ConditionExpression = expression;
    }
    const update = update_expression_1.buildUpdateExpression(params);
    if (update.expression) {
        input.UpdateExpression = update.expression;
    }
    names = Object.assign({}, names, update.names);
    values = Object.assign({}, values, update.values);
    if (Object.keys(names).length) {
        input.ExpressionAttributeNames = names;
    }
    if (Object.keys(values).length) {
        input.ExpressionAttributeValues = values;
    }
    return input;
}
exports.updateItemInput = updateItemInput;
