"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const expression_1 = require("./expression");
function queryInput(params) {
    const input = {
        TableName: params.tableName,
        IndexName: params.index,
        Limit: params.limit,
        Select: params.select,
        ConsistentRead: params.consistentRead,
        ScanIndexForward: params.order !== 'DESC',
    };
    if (params.startKey) {
        input.ExclusiveStartKey = params.startKey;
    }
    const ExpressionAttributeNames = {};
    ExpressionAttributeNames[`#${params.hashKey.name}`] = params.hashKey.name;
    const ExpressionAttributeValues = {};
    ExpressionAttributeValues[':' + params.hashKey.name] = params.hashKey.value;
    let KeyConditionExpression = `#${params.hashKey.name} = :${params.hashKey.name}`;
    if (params.rangeKey) {
        const name = params.rangeKey.name;
        ExpressionAttributeNames[`#${name}`] = name;
        let expression = '';
        const value = params.rangeKey.value;
        const arrayValue = Array.isArray(value) ? value : [value];
        const arrayValueName = arrayValue.map((_, i) => `:${name}_${i}`);
        let multiValues = true;
        if (params.rangeKey.operation === 'IN') {
            expression = `#${name} IN (${arrayValueName.join(', ')})`;
        }
        else if (params.rangeKey.operation === 'BETWEEN') {
            expression = `#${name} BETWEEN ${arrayValueName[0]} AND ${arrayValueName[1]}`;
        }
        else {
            multiValues = false;
            if (params.rangeKey.operation === 'begins_with') {
                expression = `#${name} ${params.rangeKey.operation}(:${name})`;
            }
            else {
                expression = `#${name} ${params.rangeKey.operation} :${name}`;
            }
        }
        if (multiValues) {
            arrayValue.forEach((val, i) => {
                ExpressionAttributeValues[`:${name}_${i}`] = val;
            });
        }
        else {
            ExpressionAttributeValues[`:${name}`] = value;
        }
        KeyConditionExpression += ` AND ${expression}`;
    }
    if (params.attributes && params.attributes.length) {
        const projection = expression_1.buildProjectionExpression(params.attributes, ExpressionAttributeNames);
        input.ProjectionExpression = projection.expression;
    }
    input.ExpressionAttributeNames = ExpressionAttributeNames;
    input.ExpressionAttributeValues = ExpressionAttributeValues;
    input.KeyConditionExpression = KeyConditionExpression;
    return input;
}
exports.queryInput = queryInput;
