"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
function createTableInput(table) {
    let AttributeDefinitions = [
        {
            AttributeName: table.hashKey.name,
            AttributeType: table.hashKey.type,
        }
    ];
    const KeySchema = [
        {
            AttributeName: table.hashKey.name,
            KeyType: 'HASH',
        }
    ];
    if (table.rangeKey) {
        AttributeDefinitions.push({
            AttributeName: table.rangeKey.name,
            AttributeType: table.rangeKey.type,
        });
        KeySchema.push({
            AttributeName: table.rangeKey.name,
            KeyType: 'RANGE',
        });
    }
    const LocalSecondaryIndexes = [];
    const GlobalSecondaryIndexes = [];
    if (table.indexes) {
        for (const indexInfo of table.indexes) {
            const index = formatIndex(indexInfo);
            if (indexInfo.type === 'GLOBAL') {
                GlobalSecondaryIndexes.push(index);
            }
            else {
                LocalSecondaryIndexes.push(index);
            }
            AttributeDefinitions.push({
                AttributeName: indexInfo.hashKey.name,
                AttributeType: indexInfo.hashKey.type,
            });
            if (indexInfo.rangeKey) {
                AttributeDefinitions.push({
                    AttributeName: indexInfo.rangeKey.name,
                    AttributeType: indexInfo.rangeKey.type,
                });
            }
        }
        AttributeDefinitions = helpers_1.uniqByProperty(AttributeDefinitions, 'AttributeName');
    }
    const ProvisionedThroughput = createProvisionedThroughput(table.throughput);
    return {
        TableName: table.name,
        AttributeDefinitions,
        KeySchema,
        LocalSecondaryIndexes: LocalSecondaryIndexes.length > 0 ? LocalSecondaryIndexes : undefined,
        GlobalSecondaryIndexes: GlobalSecondaryIndexes.length > 0 ? GlobalSecondaryIndexes : undefined,
        ProvisionedThroughput,
    };
}
exports.createTableInput = createTableInput;
function formatIndex(index) {
    const KeySchema = [
        {
            AttributeName: index.hashKey.name,
            KeyType: 'HASH',
        }
    ];
    if (index.rangeKey) {
        KeySchema.push({
            AttributeName: index.rangeKey.name,
            KeyType: 'RANGE',
        });
    }
    const projection = index.projection || { type: 'KEYS_ONLY' };
    const Projection = {
        ProjectionType: projection.type
    };
    if (projection.include) {
        Projection.NonKeyAttributes = projection.include;
    }
    if (index.type === 'GLOBAL') {
        return {
            IndexName: index.name,
            KeySchema,
            Projection,
            ProvisionedThroughput: createProvisionedThroughput(index.throughput),
        };
    }
    return {
        IndexName: index.name,
        KeySchema,
        Projection,
    };
}
exports.formatIndex = formatIndex;
function createProvisionedThroughput(provisionedThroughput) {
    provisionedThroughput = provisionedThroughput || {
        read: 1,
        write: 1,
    };
    return {
        ReadCapacityUnits: provisionedThroughput.read,
        WriteCapacityUnits: provisionedThroughput.write,
    };
}
