"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const expression_1 = require("./expression");
function buildUpdateExpression(params) {
    let SET = [];
    let DELETE = [];
    let REMOVE = [];
    const names = {};
    const values = {};
    if (params.delete) {
        for (const name of Object.keys(params.delete)) {
            const nameVar = expression_1.formatExpressionAttributeNameKeys(name, names);
            const valueVar = expression_1.setExpressionNameValue(values, name, params.delete[name]);
            DELETE.push(`${nameVar} ${valueVar}`);
        }
    }
    if (params.remove) {
        for (const name of params.remove) {
            REMOVE.push(expression_1.formatExpressionAttributeNameKeys(name, names));
        }
    }
    if (params.set) {
        for (const name of Object.keys(params.set)) {
            const nameVar = expression_1.formatExpressionAttributeNameKeys(name, names);
            const valueVar = buildExpressionValue(name, names, values, params.set[name]);
            SET.push(`${nameVar} = ${valueVar}`);
        }
    }
    const expressions = [];
    if (SET.length) {
        expressions.push(`SET ${SET.join(', ')}`);
    }
    if (REMOVE.length) {
        expressions.push(`REMOVE ${REMOVE.join(', ')}`);
    }
    if (DELETE.length) {
        expressions.push(`DELETE ${DELETE.join(', ')}`);
    }
    const expression = expressions.join(' ');
    return {
        expression,
        names,
        values,
    };
}
exports.buildUpdateExpression = buildUpdateExpression;
function buildExpressionValue(name, names, values, set, valueNameSuffix = '') {
    let expression = '';
    if (set.value !== undefined) {
        expression = expression_1.setExpressionNameValue(values, name + valueNameSuffix, set.value);
    }
    else if (set.path) {
        expression = expression_1.formatExpressionAttributeNameKeys(set.path, names);
    }
    else if (set.if_not_exists) {
        const pathName = expression_1.formatExpressionAttributeNameKeys(set.if_not_exists.path, names);
        const pathVar = expression_1.setExpressionNameValue(values, set.if_not_exists.path + valueNameSuffix, set.if_not_exists.value);
        expression = `if_not_exists(${pathName}, ${pathVar})`;
    }
    else if (set.list_append) {
        const list1ExpressionValue = buildExpressionValue(name, names, values, set.list_append.left, valueNameSuffix + '_lst_ppnd1');
        const list2ExpressionValue = buildExpressionValue(name, names, values, set.list_append.right, valueNameSuffix + '_lst_ppnd2');
        expression = `list_append(${list1ExpressionValue}, ${list2ExpressionValue})`;
    }
    else if (set.math) {
        const leftExpressionValue = buildExpressionValue(name, names, values, set.math.left, valueNameSuffix + '_mth1');
        const rightExpressionValue = buildExpressionValue(name, names, values, set.math.right, valueNameSuffix + '_mth2');
        expression = `${leftExpressionValue} ${set.math.operator} ${rightExpressionValue}`;
    }
    else {
        throw new Error(`Invalid UpdateExpressionSetOperand: ${set}`);
    }
    return expression;
}
exports.buildExpressionValue = buildExpressionValue;
